local ThisAddon = ...

--<<NOTES>>-----------------------------------------------------------------------------------------<<>>

    --[[

        AUTHOR(s):
            02/16 > Current: Tosaido (Creator)

        Disclaimer:
            This Code Is experimental so if any issues appear PM the author.

            This code is entirely Open source. If you would like to add it to your own addon/modify it etc 
            go right ahead. Although a reference to this addon and author would be appreciated.

            A thanks to semlars(wowinterface). I have gained a little knowledge by examining his code.

    --]]

--<<IMPORTANT STUFF>>-------------------------------------------------------------------------------<<>>

    local Visible_Plates        = {}
    local Active_Mobs           = {}

    local Core                  = CreateFrame("Frame")
    local Tooltip               = CreateFrame("GameTooltip", "KibQuestTooltip", nil, "GameTooltipTemplate")

    local WorldFrame            = WorldFrame
    local Interval_Timer        = C_Timer.After
    local Quest_Texture         = "Interface\\AddOns\\Kib_QuestMobs\\media\\questicon"
    local Ticker                = 0
    local Table_Limit           = 30
    local Player_Name

    local Num_OldPlates, total_time = 0, 0.1

--<<CREATE QUEST ICONS>>----------------------------------------------------------------------------<<>>

    local function Create_QuestIcon(Plate)                                                              --print("Created Quest Icon") 
        local Quest_Icon = Plate:CreateTexture(nil, "OVERLAY")                                   
        Quest_Icon:SetTexture(Quest_Texture)
        Quest_Icon:SetSize(16, 32)
        Quest_Icon:SetPoint("BOTTOM", Plate, "TOP")
        Quest_Icon:Hide()

        local QuestIconAnim = Quest_Icon:CreateAnimationGroup()

        local QuestIconAnim_scale = QuestIconAnim:CreateAnimation("Scale")
        QuestIconAnim_scale:SetDuration(0.7)
        QuestIconAnim_scale:SetSmoothing("OUT")
        QuestIconAnim_scale:SetFromScale(3, 3)
        QuestIconAnim_scale:SetToScale(1, 1)

        local QuestIconAnim_alpha = QuestIconAnim:CreateAnimation("Alpha")
        QuestIconAnim_alpha:SetDuration(0.5)
        QuestIconAnim_alpha:SetFromAlpha(0)
        QuestIconAnim_alpha:SetToAlpha(1)

        Quest_Icon.QuestIconAnim = QuestIconAnim
        Plate.Quest_Icon = Quest_Icon
    end

--<<SCAN MOBS FOR QUESTS>>--------------------------------------------------------------------------<<>>

    local function Scan_Quests(Plate)                                                                   --print("Scan Quests")
        local Mob_Name = Plate.Name:GetText()
        if not Mob_Name then return end

        local Guid = Active_Mobs[Mob_Name]
        if not Guid then return end

        Tooltip:SetOwner(WorldFrame, "ANCHOR_NONE")
        Tooltip:SetHyperlink("unit:" .. Guid)

        local Quest_Complete, Quest_InProgress, Quest_ObjectiveArea = nil, nil, nil
        local Quest_Found, Quest_NoObjectives = nil, nil
    
        for i = 3, Tooltip:NumLines() do
            local line = _G["KibQuestTooltipTextLeft" .. i]
            local text = line:GetText()
            local text_r, text_g, text_b = line:GetTextColor()
                                                                                                        
            if text_b == 0 and text_r > 0.99 and text_g > 0.82 then                                     --print("Quest Found")
                Quest_Found = true 

                if Quest_NoObjectives then                                                              --print("Quest Is Bar Quest")
                    Quest_ObjectiveArea = true 
                    break
                end 

                Quest_NoObjectives = true
            else
                local Unit_Name, Progress = string.match(text, "^ ([^ ]-) ?%- (.+)$")

                if Unit_Name and (Unit_Name == "" or Unit_Name == Player_Name) then
                    if Progress then
                        local current, goal = string.match(Progress, "(%d+)/(%d+)")
                        if not current or (current and goal and current == goal) then
                            Quest_NoObjectives, Quest_Complete = false, true                            --print("Quest Is Complete")
                        else
                            Quest_NoObjectives, Quest_InProgress = false, true                          --print("Quest Is Incomplete")
                        end
                    end
                end
            end
        end

        if Quest_Found then                                                                             --print("Quest Icon Shown")
            if not Plate.Quest_Icon then Create_QuestIcon(Plate) end

            local r, g, b = 1, 1, 1
            if Quest_ObjectiveArea or Quest_NoObjectives then -- The priority is - Objective Area - Incomplete Quest - Complete Quest
                r, g, b = 0.2, 0.4, 1
            elseif Quest_InProgress then
                r, g, b = 1, 1, 0
            elseif Quest_Complete then
                r, g, b = 0, 1, 0
            end

            Plate.Quest_Icon:SetVertexColor(r, g, b, 0.8)

            if not Plate.Quest_Icon:IsShown() then
                Plate.Quest_Icon:Show()
                Plate.Quest_Icon.QuestIconAnim:Play()
            end
        end
    end

--<<SET PLATE GUID>>--------------------------------------------------------------------------------<<>>

    local function FullScan_Quests()                                                                    --print("Full Scan")
        for Plate in pairs(Visible_Plates) do 
            Interval_Timer(0.01, function() Scan_Quests(Plate) end) 
        end 
    end

    local function Flush_Mobs()                                                                         --print("Flush Mobs")
        wipe(Active_Mobs)
        Ticker = 0

        for Plate in pairs(Visible_Plates) do Plate.Quest_Icon:Hide() end 
    end

    local function Set_Mob(Mob_Name, Guid)                                                              --print("Cache Mob")
        if not Active_Mobs[Mob_Name] then
            Ticker = Ticker + 1                                                                         --print("Added " .. Mob_Name .. Ticker)
            if Ticker > Table_Limit then Flush_Mobs() end

            Active_Mobs[Mob_Name] = Guid
            FullScan_Quests()
        end
    end

--<<EVENTS>>----------------------------------------------------------------------------------------<<>>

    function Core:PLAYER_TARGET_CHANGED()                                                               --print("Target Changed")
        if UnitPlayerControlled("target") then return end

        local Mob_Name = UnitName("target")     
        if Mob_Name then Set_Mob(Mob_Name, UnitGUID("target")) end      
    end     
        
    function Core:UPDATE_MOUSEOVER_UNIT()                                                               --print("MouseOver Changed")
        if UnitPlayerControlled("mouseover") then return end

        local Mob_Name = UnitName("mouseover")      
        if Mob_Name then Set_Mob(Mob_Name, UnitGUID("mouseover")) end       
    end     
        
    function Core:QUEST_LOG_UPDATE() FullScan_Quests() end                                              --print("QuestLog Updated")

    function Core:QUEST_TURNED_IN() Flush_Mobs() end                                                    --print("Quest Turned In")
        
    function Core:ADDON_LOADED(addon)                                                                   --print("Addon Loaded")
        if addon == ThisAddon then      
            Player_Name = UnitName("player")
            Core:UnregisterEvent("ADDON_LOADED")
            return
        end
    end

--<<PLATE STATE>>-----------------------------------------------------------------------------------<<>>

    local function Plate_OnShow(Plate)                                                                  --print("Plate Shown")
        Visible_Plates[Plate] = true 
        Interval_Timer(0.01, function() Scan_Quests(Plate) end)
    end

    local function Plate_OnHide(Plate)                                                                  --print("Plate Hidden")
        Visible_Plates[Plate] = nil
        if Plate.Quest_Icon then Plate.Quest_Icon:Hide() end
    end

--<<SCAN FOR PLATES>>-------------------------------------------------------------------------------<<>>

    local function Scan_Plates(_, elapsed)
        total_time = total_time + elapsed
        if total_time > 0.1 then
            total_time = 0

            local Num_CurPlates = WorldFrame:GetNumChildren()

            if Num_OldPlates < Num_CurPlates then
                for i = 1, Num_CurPlates do
                    local Plate = select(i, WorldFrame:GetChildren())
                    local name = Plate:GetName()
            
                    if not Plate.Stored and (name and string.match(name, "NamePlate")) then             --print("Plate Found")
                        local _, NameFrame = Plate:GetChildren()
                        Plate.Name = NameFrame:GetRegions()

                        Plate:HookScript("OnHide", Plate_OnHide)
                        Plate:HookScript("OnShow", Plate_OnShow)
                        if Plate:IsShown() then Plate_OnShow(Plate) end

                        Plate.Stored = true
                    end
                end

                Num_OldPlates = Num_CurPlates
            end
        end
    end

    Core:RegisterEvent("QUEST_TURNED_IN")
    Core:RegisterEvent("QUEST_LOG_UPDATE")
    Core:RegisterEvent("PLAYER_TARGET_CHANGED")
    Core:RegisterEvent("UPDATE_MOUSEOVER_UNIT") 
    Core:RegisterEvent("ADDON_LOADED")
    Core:SetScript("OnUpdate", Scan_Plates)
    Core:SetScript("OnEvent", function(self, event) Interval_Timer(0.01, function() self[event]() end) end)

----------------------------------------------------------------------------------------------------<<END>>